library(ncdf4)
#library(ggplot2)
#library(lubridate)

rm(list=ls()) ; gc()

#Directory information
setwd("../../../scratch/")
OUTPUTS <- "/scratch/"

#Useful functions
png.save <- function(x,filename,main,xlab,ylab){
  png(filename = paste(filename,".png",sep=""))
  plot(x,type="o",main=main,xlab = xlab,ylab=ylab)
  dev.off()
}

################################################################################
#Load ERA5 data
message("Loading and preprocessing data...")
nc <- nc_open("psl_5022_50km.nc")
nc_lon <- ncvar_get(nc,"lon")
nc_lat <- ncvar_get(nc,"lat")
nc_var <- ncvar_get(nc,"psl") #x,y,time

#Selection of spatial domains
gridsize_x <- abs(nc_lon[2]-nc_lon[1])
gridsize_y <- abs(nc_lat[2]-nc_lat[1])

domain1 <- list(lon <- which(nc_lon %in% seq(-15,15,by=gridsize_x)),
               lat <- which(nc_lat %in% seq(32,48,by=gridsize_y))) #red domain D1 #Spain

domain2 <- list(lon <- which(nc_lon %in% seq(-10,25,by=gridsize_x)),
                lat <- which(nc_lat %in% seq(33,45,by=gridsize_y))) #red domain D2 PEP Mediterranean

domain3 <- list(lon <- which(nc_lon %in% seq(-35,35,by=gridsize_x)),
                lat <- which(nc_lat %in% seq(20,70,by=gridsize_y))) #red domain D3 #Europe

domain <- domain3 ; dom = "D3_50km"
nc_var <- nc_var[domain[[1]],domain[[2]],] ; gc()

#Organize and Standardize data
var <- matrix(nc_var,dim(nc_var)[1]*dim(nc_var)[2],dim(nc_var)[3]) #space,time
rm(nc_var) ; gc()

message("Standardizing...")
var.sc <- scale(t(var),center=T,scale=T) #var[,1:365]
message("Done.")

################################################################################
#Data reduction
message("Data reduction by means of EOFs...")
RR=cor(var.sc)
DD=svd(RR)

U=DD$u ; V=DD$v
dv=DD$d ; dd=diag(DD$d)

FF=U%*%dd^0.5 #loadings matrix
A=dv^(-0.5)
AA=diag(A)
Zs=var.sc%*%U%*%AA  #CPs matrix

SumDiag=sum(dv)
VAR=(dv/SumDiag)*100 #explained variance

k=which(cumsum(VAR)>90)[1]
rm(RR,DD,U,V,A,AA,dd,FF) ; gc()
message("Done.")

################################################################################
#Clustering procedure
message("Performing clustering...")
input.data <- Zs[,1:k]
ward_clustering <- hclust(d = dist(input.data),method = "ward.D2")

save(ward_clustering,file=paste(OUTPUTS,"WardCluster_Dendrogram_",dom,".Rdata",sep=""))

#Sensibility to the number of clusters
wss <- (nrow(input.data)-1)*sum(apply(input.data,2,var))
for(i in 2:15){
wss.sub <- NA
  for(j in 1:i){
  sub.data <- input.data[which(cutree(ward_clustering,k=i)==j),]
  wss.sub[j] <- (nrow(sub.data)-1)*sum(apply(sub.data,2,var))
  }
  wss[i] <- sum(wss.sub)
}
message("Done.")

pseudoF = function(X, k,wss){
  nk = length(k)
  n = nrow(X)
  T = sum(scale(X,scale=F)^2)
  W = rep(T, nk)
  for (i in 1:nk){
    set.seed(12345)
    #cli = kmeans(X, k[i],nstart=ns)
    W[i] = wss[i]
  }
  pF = ((T-W)/(k-1))/(W/(n-k))
  return(list(k=k, W=W, pF=pF))
}

#Initial assessment
message("Plotting initial assessment results...")
png.save(wss,filename = paste("wss_",dom,sep=""),main=paste("Ward PSL ",dom," 1950-2022",sep=""),
         xlab="Number of patterns",ylab="Total within sum of squares")

png(filename = paste(OUTPUTS,"pseudo-F PSL",dom," 1950-2022",".png",sep=""),width=3000,height=1500,res=300)
plot(pseudoF(input.data,1:15,wss)$pF,type="o",main="Pseudo-F",xlab="Number of patterns",ylab="Pseudo-F")
dev.off()

png(filename = paste(OUTPUTS,"Dendogram Ward PSL",dom," 1950-2022",".png",sep=""),width=3000,height=1500,res=300)
  plot(ward_clustering,  xlab = "Height", 
       edgePar = list(col = 2:3, lwd = 2:1), xaxt='n')
  rect.hclust(tree = ward_clustering, k = 4, which = 1:4, border = 1:4, cluster = cutree(tree = ward_clustering, k = 4))
  rect.hclust(tree = ward_clustering, k = 8, which = 1:8, border = 1:8, cluster = cutree(tree = ward_clustering, k = 8))
dev.off()

message("Done.")

################################################################################
